@@ -15,7 +15,8 @@ from typing import List, Optional, Tuple, Union
 
 import libconf
 
-from swugenerator import __about__, generator
+from swugenerator import __about__, generator, signer
+
 from swugenerator.swu_sign import SWUSignCMS, SWUSignCustom, SWUSignPKCS11, SWUSignRSA
 
 
@@ -27,15 +28,6 @@ class InvalidSigningOption(ValueError):
     """Raised when an invalid signing option is passed via command line"""
 
 
-class UpdateAction(argparse.Action):
-    def __call__(self, parser, namespace, values, option_string=None):
-        cfg = getattr(namespace, self.dest, None)
-        if cfg is None:
-            cfg = {}
-        cfg.update(*values)
-        setattr(namespace, self.dest, cfg)
-
-
 def extract_keys(keyfile: str) -> Tuple[Optional[str], Optional[str]]:
     """Extracts encryption key and initialization vector (IV)
 
@@ -102,7 +94,7 @@ def parse_signing_option(
     CMS,<private key>,<certificate used to sign>
     RSA,<private key>,<file with password>
     RSA,<private key>
-    PKCS11,<pin>[,<module>]
+    PKCS11,<pin>[,<module>,<slot>,<id>]
     CUSTOM,<custom command>
 
     Args:
@@ -143,12 +135,15 @@ def parse_signing_option(
         # Format : RSA,<private key>
         return SWUSignRSA(sign_parms[1], None)
     if cmd == "PKCS11":
-        # Format : PKCS11,<pin>[,<module>]
-        if len(sign_parms) not in (2, 3) or not all(sign_parms[0:2]):
-            raise InvalidSigningOption("PKCS11 requires pin and optional module path")
+        # Format : PKCS11,<pin>[,<module>,<slot>,<id>]
+        if len(sign_parms) not in range(2, 6) or not all(sign_parms[0:2]):
+            raise InvalidSigningOption("PKCS11 requires pin and optional parameters such as module path, slot or id")
         pin = sign_parms[1]
-        module = sign_parms[2] if len(sign_parms) == 3 else None
-        return SWUSignPKCS11(pin, module)
+        module = sign_parms[2] if len(sign_parms) > 2 else None
+        slot   = sign_parms[3] if len(sign_parms) > 3 else None
+        obj_id = sign_parms[4] if len(sign_parms) > 4 else None
+
+        return SWUSignPKCS11(pin, module, slot, obj_id)
     if cmd == "CUSTOM":
         # Format : CUSTOM,<custom command>
         if len(sign_parms) < 2 or not all(sign_parms):
@@ -210,6 +205,14 @@ def create_swu(args: argparse.Namespace) -> None:
     swu.process()
     swu.close()
 
+def sign_swu(args: argparse.Namespace) -> None:
+    swu = signer.SWUSigner(
+        args.swu_in_file,
+        args.swu_file,
+        args.sign
+    )
+    swu.process()
+    swu.close()
 
 def parse_args(args: List[str]) -> None:
     """Sets up arguments for swugenerator and parses commandline args
@@ -271,7 +274,7 @@ def parse_args(args: List[str]) -> None:
                  -g, --engine ENGINE       OpenSSL engine to use for signing (e.g., pkcs11)
                  -f, --keyform KEYFORM     Key format to use for signing (e.g., engine)
             RSA,<private key>,<file with password if any>
-            PKCS11,<pin>[,<module>]
+            PKCS11,<pin>[,<module>,<slot>,<id>]
             CUSTOM,<custom command> """
         ),
     )
@@ -279,9 +282,9 @@ def parse_args(args: List[str]) -> None:
     parser.add_argument(
         "-s",
         "--sw-description",
-        required=True,
+        required=False,
         type=lambda p: Path(p).resolve(),
-        help="sw-description template",
+        help="sw-description template, required for the create command",
     )
 
     parser.add_argument(
@@ -309,13 +312,10 @@ def parse_args(args: List[str]) -> None:
         help="SWU output file",
     )
 
-    parser.register("action", "update", UpdateAction)
     parser.add_argument(
         "-c",
         "--config",
         default={},
-        action="update",
-        nargs="*",
         type=parse_config_file,
         help="configuration file",
     )
@@ -347,10 +347,23 @@ def parse_args(args: List[str]) -> None:
     )
     create_subparser = subparsers.add_parser("create", help="creates a SWU file")
     create_subparser.set_defaults(func=create_swu)
-
+    sign_subparser = subparsers.add_parser("sign", help="signs an existing SWU file provided by -i")
+    sign_subparser.set_defaults(func=sign_swu)
+    sign_subparser.add_argument(
+        "-i",
+        "--swu-in-file",
+        required=True,
+        type=Path,
+        help="SWU input file to be signed, required for the sign command",
+    )
     args = parser.parse_args(args)
     if hasattr(args, 'sign') and args.sign:
         args.sign = parse_signing_option(args.sign, args.engine, args.keyform)
+    if args.func == create_swu and not args.sw_description:
+        parser.error(
+            "the following arguments are required: -s/--sw-description"
+        )
+
     args.func(args)
 
 
